﻿using Brilliant.Data.Common;
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Linq;
using System.Text;

namespace Brilliant.Data.Provider
{
    /// <summary>
    /// DataProvider基类
    /// </summary>
    public abstract class DataProviderBase : IDataProvider
    {
        private ConnectionInfoBase _connInfo;

        /// <summary>
        /// 当前连接
        /// </summary>
        protected ConnectionInfoBase ConnInfo
        {
            get { return _connInfo; }
        }

        /// <summary>
        /// 构造器
        /// </summary>
        public DataProviderBase() { }

        /// <summary>
        /// 构造器
        /// </summary>
        /// <param name="connectionString">连接字符串</param>
        public DataProviderBase(ConnectionInfoBase connInfo)
        {
            this._connInfo = connInfo;
        }

        /// <summary>
        /// 变更连接字符串
        /// </summary>
        /// <param name="connectionString">连接字符串</param>
        public void ChangeConnectionString(string connectionString)
        {
            ConnInfo.ConnectionString = connectionString;
        }

        /// <summary>
        /// 检测连接是否可用
        /// </summary>
        /// <param name="connInfo">连接</param>
        /// <returns>true:可用 false不可用</returns>
        public bool CheckConnection(ConnectionInfo connInfo)
        {
            if (String.IsNullOrEmpty(connInfo.ConnectionString))
            {
                return false;
            }
            this._connInfo = connInfo;
            using (DbConnection conn = GetConnection())
            {
                try
                {
                    if (conn.State != ConnectionState.Open)
                    {
                        conn.Open();
                    }
                    return true;
                }
                catch
                {
                    this._connInfo = null;
                    return false;
                }
            }
        }

        /// <summary>
        /// 返回Command对象
        /// </summary>
        private DbCommand GetCommand(DbConnection conn, SQL sql)
        {
            if (conn.State != ConnectionState.Open)
            {
                conn.Open();
            }
            DbCommand cmd = GetCommand();
            cmd.Connection = conn;
            cmd.CommandText = sql.CmdText;
            cmd.CommandType = sql.CmdType;
            if (sql.Parameters != null)
            {
                cmd.Parameters.Clear();
                cmd.Parameters.AddRange(sql.Parameters);
            }
            return cmd;
        }

        /// <summary>
        /// 执行查询指令返回DataSet对象
        /// </summary>
        /// <param name="sql">查询指令</param>
        /// <returns>DataSet对象</returns>
        public DataSet ExecDataSet(SQL sql)
        {
            using (DbConnection conn = GetConnection())
            {
                DbCommand cmd = GetCommand(conn, sql);
                DbDataAdapter da = GetDataAdapter();
                da.SelectCommand = cmd;
                DataSet ds = new DataSet();
                da.Fill(ds);
                cmd.Parameters.Clear();
                return ds;
            }
        }

        /// <summary>
        /// 执行查询指令返回DataReader对象
        /// </summary>
        /// <param name="sql">查询指令</param>
        /// <returns>DataReader对象</returns>
        public IDataReader ExecDataReader(SQL sql)
        {
            DbConnection conn = GetConnection();
            DbCommand cmd = GetCommand(conn, sql);
            IDataReader dataReader = cmd.ExecuteReader(CommandBehavior.CloseConnection);
            cmd.Parameters.Clear();
            return dataReader;
        }

        /// <summary>
        /// 执行查询指令返回第一行第一列的值
        /// </summary>
        /// <param name="sql">查询指令</param>
        /// <returns>第一行第一列的值</returns>
        public object ExecScalar(SQL sql)
        {
            using (DbConnection conn = GetConnection())
            {
                DbCommand cmd = GetCommand(conn, sql);
                object result = cmd.ExecuteScalar();
                cmd.Parameters.Clear();
                //新增对于DBNull的判定，将DBnull转换为null以供逻辑判断（2014-09-05）
                return result == DBNull.Value ? null : result;
            }
        }

        /// <summary>
        /// 执行查询指令返回受影响行数
        /// </summary>
        /// <param name="sql">查询指令</param>
        /// <returns>受影响行数</returns>
        public int ExecNonQuerry(SQL sql)
        {
            using (DbConnection conn = GetConnection())
            {
                DbCommand cmd = GetCommand(conn, sql);
                int result = cmd.ExecuteNonQuery();
                cmd.Parameters.Clear();
                return result;
            }
        }

        /// <summary>
        /// 返回一个新的Connection实例
        /// </summary>
        /// <returns>Connection实例</returns>
        protected abstract DbConnection GetConnection();

        /// <summary>
        /// 返回一个新的Command实例
        /// </summary>
        /// <returns>Command实例</returns>
        protected abstract DbCommand GetCommand();

        /// <summary>
        /// 返回一个新的DataAdapter实例
        /// </summary>
        /// <returns>DataAdapter实例</returns>
        protected abstract DbDataAdapter GetDataAdapter();
    }
}
